#######################################################################################
# This file contains R definitions and functions relating to the phenotype trait
# analysis done in preparation for the root RNAseq Sep 2011 experiment.
# Author: Ted Toal
# Lab: Brady
#######################################################################################

cat("Including Include_phenoRNAseqRootSep2011.R\n")

#######################################################################################
# Phenotype analysis directory paths.
#######################################################################################

# Cell counting subfolder.
cellCountingDir = myIfElse(exists("MY_cellCountingDir"), MY_cellCountingDir,
    paste(phenoRNAseqRootSep2011Dir, "Cell_Counting", sep=PATHSEP))

# Cell counting analysis output folder and output file and path prefix.
cellCountingOutFolder = "11Feb15" # "23Apr12"
cellCountingOutDir = paste(cellCountingDir, cellCountingOutFolder, sep=PATHSEP)
cellCountingOutFilePfx = "ILsCellCounting_MinRoots3_NoPaper_Data032312_"
cellCountingOutPathPfx = paste(cellCountingOutDir, cellCountingOutFilePfx, sep=PATHSEP)

# Root length/gravity subfolder.
rootLenAngDir = myIfElse(exists("MY_rootLenAngDir"), MY_rootLenAngDir,
    paste(phenoRNAseqRootSep2011Dir, "RootLength_Gravity", sep=PATHSEP))

# Root length/gravity analysis output folder and output file and path prefix.
rootLenAngOutFolder = "11Feb15" # "12Mar13"
rootLenAngOutDir = paste(rootLenAngDir, rootLenAngOutFolder, sep=PATHSEP)
rootLenAngOutFilePfx = "LenAng_MinSigDt2_MinPtsDt3_NoPaper_Data110111_"
rootLenAngOutPathPfx = paste(rootLenAngOutDir, rootLenAngOutFilePfx, sep=PATHSEP)

#######################################################################################
# Phenotype analysis file paths and names.
#######################################################################################

# This file contains three columns: trait, unit, desc, and should be read with the
# trait as rownames of a data frame.  It contains units and description for each trait.
traitsFile = paste(phenoRNAseqRootSep2011Dir, "Traits_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)

# The phenotype analysis produced several files that we will provide paths to here:
#   traitDescsFile: contains descriptions of phenotype data, with two columns: "trait", "desc".
#   traitUnitsFile: contains units of phenotype data, with two columns: "trait", "unit".
#   traitGroupsFile: contains groupings of similar traits, with two columns: "group", "trait".
#   traitMeansFile: contains analyzed phenotype data, with four columns:
#                   "trait", "genotype", "mean", and "isLSmean".
#   traitStdErrFile: contains standard errors of means for comparison of each trait in
#                   each IL to M82, with rows being IL names and 3 columns: "trait",
#                   "genotype", and "sem".
#   traitPvalsFile: contains p-values for comparison of each trait in each IL to M82,
#                   with rows being IL names and 3 columns: "trait", "genotype", and "pvalue".
#   traitStarsFile: contains number of stars assigned to each trait in each IL based
#                   on the q-value, with rows being IL names and columns being trait
#                   names.
traitDescsFile = paste(phenoRNAseqRootSep2011Dir, "TraitDescs_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitUnitsFile = paste(phenoRNAseqRootSep2011Dir, "TraitUnits_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitGroupsFile = paste(phenoRNAseqRootSep2011Dir, "TraitGroups_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitMeansFile = paste(phenoRNAseqRootSep2011Dir, "TraitMeans_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitPvalsFile = paste(phenoRNAseqRootSep2011Dir, "TraitPvals_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitStdErrFile = paste(phenoRNAseqRootSep2011Dir, "TraitStdErr_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitPlusFile = paste(phenoRNAseqRootSep2011Dir, "TraitPlus_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitQvalsFile = paste(phenoRNAseqRootSep2011Dir, "TraitQvals_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)
traitStarFile = paste(phenoRNAseqRootSep2011Dir, "TraitStar_CC11Feb15_RL11Feb15.tsv", sep=PATHSEP)

# Following is derived from heatmap.
traitStarsFile = paste(phenoRNAseqRootSep2011Dir, "TraitStars.tsv", sep=PATHSEP)

#######################################################################################
# End of file.
#######################################################################################
Sourced.Include_phenoRNAseqRootSep2011 = TRUE
cat("  Include_phenoRNAseqRootSep2011.R included\n")
