#######################################################################################
# This file contains R definitions and functions relating to making of Heat Maps.
# Author: Ted Toal
# Lab: Brady
#######################################################################################

cat("Including Include_Heatmaps.R\n")

# Load cluster library, unless Include_RootRsourceFile.R is being used, in which case
# require it.
{
if (exists("Sourced.Include_RootRsourceFile"))
    requirePackage("cluster", "Include_Heatmaps")
else
    addThisLibrary("cluster")
}

#######################################################################################
# Fill in the current plot region with the specified colors.
#######################################################################################
colorPlotArea = function(mainPlotRgnColor="black", marginColor="red")
    {
    usr = par("usr")
    rect(usr[1], usr[3], usr[2], usr[4], col=mainPlotRgnColor, border=NA)
    plt = par("plt")
    x1 = usr[1] - plt[1]*(usr[2]-usr[1])/(plt[2]-plt[1])
    x2 = usr[2] + (1-plt[2])*(usr[2]-usr[1])/(plt[2]-plt[1])
    y1 = usr[3] - plt[3]*(usr[4]-usr[3])/(plt[4]-plt[3])
    y2 = usr[4] + (1-plt[3])*(usr[4]-usr[3])/(plt[4]-plt[3])
    sv = par(xpd=TRUE)
    rect(x1, y1, usr[1], y2, col=marginColor, border=NA)
    rect(usr[1], y1, usr[2], usr[3], col=marginColor, border=NA)
    rect(usr[2], y1, x2, y2, col=marginColor, border=NA)
    rect(usr[1], usr[4], usr[2], y2, col=marginColor, border=NA)
    par(xpd=sv)
    }

#######################################################################################
# Make and return a vector of nice heat map colors composed of shades of one color for
# negative values and shades of another for positive values, and a third color for zero
# values.  The number of minus and plus colors is specified as Nplus and Nminus.
#######################################################################################
makeHeatmapColors = function(zeroColorName, plusColorName, Nplus, minusColorName="black", Nminus=0)
    {
    Ncolors = Nminus+Nplus+1
    plusColor = as.data.frame(t(col2rgb(plusColorName)/255))
    zeroColor = as.data.frame(t(col2rgb(zeroColorName)/255))
    minusColor = as.data.frame(t(col2rgb(minusColorName)/255))
    heatmapColors = c()
    if (Nminus > 0)
        {
        for (i in Nminus:1)
            {
            k = i/Nminus
            col = (k*minusColor + (1-k)*zeroColor)
            heatmapColors = c(heatmapColors, rgb(col$red, col$green, col$blue))
            }
        }
    heatmapColors = c(heatmapColors, rgb(zeroColor$red, zeroColor$green, zeroColor$blue))
    if (Nplus > 0)
        {
        for (i in 1:Nplus)
            {
            k = i/Nplus
            col = (k*plusColor + (1-k)*zeroColor)
            heatmapColors = c(heatmapColors, rgb(col$red, col$green, col$blue))
            }
        }
    return(heatmapColors)
    }

#######################################################################################
# This is heatmap.2() of package gplots, modified as follows by Ted Toal:
#   (See ?heatmap.2 for initial description of arguments)
#   New arguments:
#       heatmapAreaSize: size (width, height) of heatmap area in percent of page size.  The
#           remainder of the page area after subtracting the outer margins (argument margins)
#           and this argument's values from 100% is used as the size of the 2nd layout row
#           and column (see arguments lmat, lhei, lwid below), which are used to plot the
#           optional key and optional dendrograms.  You must set this argument small enough
#           to leave enough space for those.  See lmat below; this is the size of region 1.
#       heatmapSize: This is a two-element vector, the width and height of the heatmap image
#           within the heatmap plot area.  The units are percent of the heatmap plot area,
#           given by heatmapAreaSize.  Within the heatmap plot area, the heatmap image of
#           size heatmapSize is plotted with the space around it depicted in the layout
#           pictorial below.
#       rowSideBarSize, colSideBarSize: Each of these is a two-element vector, first element
#           is size (width or height) of the side bar, second element is amount of space to
#           put between the heatmap and sidebar.  Numbers are percent of heatmapAreaSize,
#           and these sizes reduce the actual heatmap size.  These default to c(0,0) if
#           RowSideColors or ColSideColors are NULL so no side bar is drawn.
#       lftLabRow, rgtLabRow: TRUE to put row labels left/right of heatmap.  If TRUE, you
#           may need to specify the following arguments to adjust the margin to accommodate.
#           If both labels and row side bar are used, the side bar is next to the heatmap
#           and the labels are next to the side bar.
#       lftLabSize, rgtLabSize: Each of these is a two-element vector, first element is
#           width of left or right label area, second element is amount of space to put
#           between the sidebar/heatmap and the label.  Numbers are percent of heatmapAreaSize,
#           and these sizes reduce the actual heatmap size.  These default to c(0,0) if
#           lftLabRow or rgtLabRow is FALSE so no labels are drawn.
#       lftLabs, rgtLabs: Vectors of label text for left and right sides.  These default to
#           the value of the labRow argument.
#       btmLabCol, topLabCol: TRUE to put column labels below/above heatmap.  If TRUE, you
#           may need to specify the following arguments to adjust the margin to accommodate.
#           If both labels and column side bar are used, the side bar is next to the heatmap
#           and the labels are next to the side bar.
#       btmLabSize, topLabSize: Each of these is a two-element vector, first element is
#           height of bottom or top label area, second element is amount of space to put
#           between the sidebar/heatmap and the label.  Numbers are percent of heatmapAreaSize,
#           and these sizes reduce the actual heatmap size.  These are ignored if btmLabCol
#           or topLabCol is FALSE so no labels are drawn.
#       btmLabs, topLabs: Vectors of label text for bottom and top sides.  These default to
#           the value of the colRow argument.
#       colLabRow, colLabCol: vector of colors for row and column labels, default "black".
#       srtLabRow, srtLabCol: par(srt) value for row and column labels, default 0, 85.
#       rowDendroSize, colDendroSize: Each of these is a two-element vector, first element
#           is size (width or height) of the dendrogram, second element is amount of space to
#           put between the heatmap and dendrogram.  Numbers are percent of the dendrogram
#           plotting area width or height (see lmat below).  These are ignored if RowV or
#           ColV specify that no dendrogram is to be drawn.  The axis labels xlab and ylab
#           are plotted in the space that remains in the dendrogram row or column after
#           subtracting off these numbers, so they should not total 100% if you are using
#           xlab or ylab.
#       mainKey, xlabKey, ylabKey: Title and axis labels of key.
#       cex.mainKey, cex.axisKey, cex.xlabKey, cex.ylabKey: cex values for title and axis labels of key.
#       line.xlabKey, line.ylabKey: margin line numbers for axis labels of key.
#       lvKey: vector of x-values of color breaks at color and histogram x-axis label positions.
#           Default uses pretty() on breaks and x.
#       xvalKey: vector of labels for color key and histogram x-axis.  Default uses lvKey values.
#       key: This may be logical TRUE or FALSE, or it may be a number: 2, 3, or 4.  If FALSE,
#           no key is drawn.  If TRUE, a key is drawn in region 4, see lmat below. If it is a
#           number 2..4, a key is drawn in that region, see lmat below.  The size of the key
#           plot area depends on which region the key is drawn in.  If region 1, heatmapAreaSize
#           gives the region size.  If regions 2, 3, or 4, the size is determined by both
#           heatmapAreaSize and the outer margin sizes, see lmat below.
#       keySize: This is a two-element vector, the width and height of the key color image
#           within the key plot area.  It was called keySize in heatmap.2 and was only 1
#           element in size.  The units are percent of the key plot area (see lmat below).
#           Within the key plot area, the key color image of size keySize is plotted with
#           space left and above it (within the key plot area) given by keySpace.  The
#           remaining key plot area space must be enough to plot the key axis labels and
#           axes.  If these don't fit and are truncated, you need to either reduce keySize
#           or keySpace or heatmapAreaSize or the margins.  It is up to you to set the sizes
#           so that keySpace doesn't run into the main heatmap labels or dendrograms.
#       keySpace: This is a two-element vector, the amount of space to the left and above
#           the key color image, in units of percent of the size of the key plot area.
#           The y-axis and its label are plotted within the keySpace[1] area and the key
#           title is plotted within the keySpace[2] area, while the x-axis and its labels
#           are plotted below the key color image in whatever key plot area space remains.
#           See heatmapAreaSize and keySize for more info.
#       frame: TRUE to draw a frame around the main heatmap and key heatmap.
#       col.frame, lwd.frame: color and width of the frame.
#       sub: Subtitle.
#       cex.main, cex.sub, cex.xlab, cex.ylab: Title/subtitle/xlab/ylab text magnification factors.
#       col.main, col.sub, col.xlab, col.ylab: Title/subtitle/xlab/ylab text colors.
#       line.main, line.sub, line.xlab, line.ylab: Title/subtitle/xlab/ylab text margin line.
#       cex.key: Key text magnification factor.
#       plot.Heatmap: function to call after plotting the heatmap, NULL if none.  Arguments
#           are xlim, ylim, nr (number rows), nc (number columns), and more can be added if needed.
#       showDebugInfo: TRUE to show debugging info.
#   Argument changes:
#       margins: This is the size of the outer margin region.  It is now specified in
#           units of percent of page size instead of lines.  The length of the margins
#           vector must be 1, 2, or 4.  If 1, it is used for all four margins.  If 2, the
#           first is used for top and bottom, second for left and right.  If 4, these are
#           bottom, left, top, right in that order, unless elements are named with those
#           names, in which case order doesn't matter.  These margins are set as the OUTER
#           MARGINS (par(omi or oma)) of the plot area, and nothing is placed in these
#           margins EXCEPT the plot title (arg main) and subtitle (arg sub), so make the
#           top margin large enough for these.
#       xlab, ylab: if these are not NULL, they are placed just outside the row/column
#           dendrograms (if any) or heatmap and its sidebars and column labels.
#       lmat, lhei, lwid: As before, lmat is the layout matrix, lwid is the width of the
#           layout columns, and lhei is the height of the layout rows.  Units are
#           arbitrary (relative to each other).  lmat default is this:
#           -------------------------
#           |  Outer Margin Region  |
#           |     +-----+-----+     |
#           |     |  4  |  3  |     |
#           |     +-----+-----+     |
#           |     |  2  |  1  |     |
#           |     +-----+-----+     |
#           |                       |
#           -------------------------
# Zeroes are in the lmat cells of the outer margin region.  The non-zero numbers in lmat mark
# display regions where the following elements are plotted:
#   1: Heatmap with optional side bars, optional row labels on left and/or right side,
#       and optional column labels on top and/or bottom side.  These are left and top
#       justified, so any space remaining in the heatmap plot area serves as space between
#       the heatmap lower and right labels and the page boundary.  The size of this region
#       is specified using the heatmapAreaSize argument.
#   2: Optional row dendrogram, right justified, and optional y-axis label (ylab) left of it.
#       This region's height is heatmapAreaSize[2] and width is 100-outerMargins-heatmapAreaSize[1].
#   3: Optional column dendrogram, bottom justified, and optional x-axis label (xlab) above it.
#       This region's width is heatmapAreaSize[1] and height is 100-outerMargins-heatmapAreaSize[2].
#   4: Key (including key color image, axes with tick marks and tick labels, and axis labels)
#       IF the key argument is TRUE or 4 (otherwise, this region is empty).  When the key is
#       in region 4, its elements are left and top justified, except that keySpace provides
#       space to the left and above, so any space remaining in the key plot area (equal to
#       100%-keySpace-keySize-axes-labels) serves as space between the key and the optional
#       dendrograms or top labels.
#
# Region 1 containing the heatmap is further subdivided by heatmapSize as follows.
#                       Layout of Region 1 above:
#           +-----------------------+-----+-----------+------
#           |                       |  k  |           |     |
#           |                       +-----+           |     |
#           |                       |  g  |           |     |
#           |                       +-----+           |     |
#           |                       |  e  |           |     |
#           |                       +-----+           |     |
#           |                       |  c  |           |     |
#           +-----+-----+-----+-----+-----+-----+-----+     |
#           |  j  |  f  |  d  |  b  |  a  |  h  |  l  |  n  |
#           +-----+-----+-----+-----+-----+-----+-----+     |
#           |                       |  i  |           |     |
#           |                       +-----+           |     |
#           |                       |  m  |           |     |
#           +-----------------------+-----+-----------+     |
#           |                          o                    |
#           -------------------------------------------------
#                   (start at "a" and spiral outwards)
# The letters mark display regions where the following elements are plotted:
#   a: Heatmap image is plotted here.  Size is specified by the heatmapSize argument.
#       The cell aspect ratio is affected by both heatmapSize and number of heatmap rows/columns.
#   b: Space between the heatmap and row sidebar.  Width is rowSideBarSize[2] even if no sidebar.
#   c: Like b but for column sidebar.  Height is colSideBarSize[2].
#   d: Row sidebar image goes here.  Width is rowSideBarSize[1].
#   e: Like d but for column sidebar.  Height is colSideBarSize[1].
#   f: Space between row sidebar and left row labels.  Width is lftLabSize[2] even if no labels.
#   g: Like f but for top column labels.  Height is topLabSize[2].
#   h: Like f but for right row labels.  Width is rgtLabSize[2].
#   i: Like f but for bottom column labels.  Height is btmLabSize[2].
#   j: Left row labels.  Width is lftLabSize[1].
#   k: Like j but top column labels.  Height is topLabSize[1].
#   l: Like j but right row labels.  Width is rgtLabSize[1].
#   m: Like j but bottom column labels.  Height is btmLabSize[1].
#   n: Space not consumed by the regions left of this appears on right side.
#       Width is 100-(j+f+d+b+a+h+l).  Those widths must sum to no more than 100!
#   o: Like n but remaining space appears on bottom side.  Height is 100-(k+g+e+c+a+i+m).
#
# The plot title and subtitle are placed in the outer margin region, as far away from the
# edge as possible.
# After setting the outer margins using the margins argument, the size of the inner area
# must be broken down into two columns and two rows.  This is done using the heatmapAreaSize
# argument, which gives the size of cell 1 and thus of row 3 and column 3 above.  The
# remaining page space goes to row 2 and column 2.
# If you wish to specify lmat yourself instead of using the default, you must specify
# all of lmat/lhei/lwid, you cannot specify just one of these three.  They are not
# altered from what you specify.
#
# Return value: a list, invisibly, with elements:
#   hBnds
#   rowInd
#   colInd
#   call
#   rowMeans
#   rowSDs
#   colMeans
#   colSDs
#   carpet
#   rowDendrogram
#   colDendrogram
#   breaks
#   col
#   vline
#   hline
#   colorTable
#######################################################################################
tedsHeatmap = function(x, Rowv=TRUE, Colv=if (symm) "Rowv" else TRUE, 
    distfun=dist, hclustfun=hclust, dendrogram=c("both", "row", "column", "none"),
    margins=c(bottom=8,left=8,top=15,right=8),
    heatmapAreaSize=c(60,60), heatmapSize=c(60,65),
    rowDendroSize=c(60,0), colDendroSize=c(60,0),
    symm=FALSE, scale=c("none", "row", "column"), na.rm=TRUE, revC=identical(Colv, "Rowv"),
    breaks, symbreaks=min(x < 0, na.rm=TRUE) || scale != "none", col="heat.colors",
    frame=FALSE, col.frame="black", lwd.frame=1,
    colsep, rowsep, sepcolor="white", sepwidth=c(0.05, 0.05),
    cellnote, notecex=0.6, notecol="black", na.color=par("bg"),
    trace=c("column", "row", "both", "none"), tracecol="cyan",
    hline=median(breaks), vline=median(breaks), linecol=tracecol, cexRow=NULL, cexCol=NULL,
    RowSideColors=NULL, ColSideColors=NULL,
    rowSideBarSize=ifelse(!is.null(RowSideColors), c(2.5,0.5), c(0,0)),
    colSideBarSize=ifelse(!is.null(ColSideColors), c(2.5,0.5), c(0,0)),
    labRow=NULL, colLabRow="black", srtLabRow=0, lftLabRow=FALSE, rgtLabRow=TRUE,
    lftLabSize=ifelse(lftLabRow, c(14,0.5), c(0,0)), rgtLabSize=ifelse(rgtLabRow, c(14,0.5), c(0,0)),
    labCol=NULL, colLabCol="black", srtLabCol=85, btmLabCol=TRUE, topLabCol=FALSE,
    btmLabSize=ifelse(btmLabCol, c(14,0.5), c(0,0)), topLabSize=ifelse(topLabCol, c(14,0.5), c(0,0)),
    lftLabs=NULL, rgtLabs=NULL, btmLabs=NULL, topLabs=NULL,
    key=TRUE, keySize=c(75,25), keySpace=c(15,25),
    density.info=c("histogram", "density", "none"),
    denscol=tracecol, symkey=min(x < 0, na.rm=TRUE) || symbreaks, densadj=0.25,
    main=NULL, sub=NULL,
    cex.main=par("cex.main"), cex.sub=par("cex.sub"), cex.xlab=0.8, cex.ylab=0.8,
    col.main=par("col.main"), col.sub=par("col.sub"), col.xlab="black", col.ylab="black",
    line.main=2, line.sub=0.5, line.xlab=0.5, line.ylab=0.5,
    xlab=NULL, ylab=NULL,
    lmat=NULL, lhei=NULL, lwid=NULL,
    mainKey=NULL, xlabKey=NULL, ylabKey=NULL,
    cex.key=0.7, cex.mainKey=0.9, cex.axisKey=0.9, cex.xlabKey=0.7, cex.ylabKey=0.7,
    line.xlabKey=2, line.ylabKey=2,
    lvKey=NULL, xvalKey=NULL,
    plot.Heatmap=NULL,
    showDebugInfo=FALSE, ...)
    {
    # Scale a vector of numbers to be between 0 and 1, 0 if equal to low, 1 if equal to high.
    scale01 = function(x, low=min(x), high=max(x))
        {
        x = (x - low)/(high - low)
        return(x)
        }

    # Convert a page- or plot-percent into number of inches.  One of w (width) or h (height)
    # must be specified.  If pct2 is specified, it is an additional percent factor to
    # include in the calculation (e.g. labelWidth = label%/100 * heatmap%/100 * devWidth).
    devSizeInches = dev.size("in")
    pctToInch = function(w=NA, h=NA, pct2=100)
        {
        pct2 = pct2/100
        if (!all(is.na(w)))
            return(pct2*devSizeInches[1]*w/100)
        return(pct2*devSizeInches[2]*h/100)
        }

    # Convert a page- or plot- percent into a delta-user-coordinates.  Arguments are like
    # pctToInch.
    pctToUser = function(w=NA, h=NA, pct2=100)
        {
        pct2 = pct2/100
        if (!all(is.na(w)))
            {
            userX = par("usr")[1:2]
            pltX = par("plt")[1:2]
            user100pct = (userX[2]-userX[1])/(pltX[2]-pltX[1])
            return(pct2*user100pct*w/100)
            }
        userY = par("usr")[3:4]
        pltY = par("plt")[3:4]
        user100pct = (userY[2]-userY[1])/(pltY[2]-pltY[1])
        return(pct2*user100pct*h/100)
        }

    # Store return values in this list, which IS the return value.
    retval = list()
    # scale indicates if row/column values are to be scaled to N(0, 1).
    scale = if (symm && missing(scale)) "none" else match.arg(scale)
    # dendrogram indicates whether or not to show row/column dendrograms.
    dendrogram = match.arg(dendrogram)
    # trace indicates whether or not to show a trace line in the heatmap.
    trace = match.arg(trace)
    # density.info indicates whether to do density plot, histogram, or none for key.
    density.info = match.arg(density.info)
    # col is heatmap colors.
    if (length(col) == 1 && is.character(col)) 
        col = get(col, mode="function")
    # breaks is number of color bins, or vector of actual color bins.
    if (!missing(breaks) && (scale != "none")) 
        warning("Using scale=\"row\" or scale=\"column\" when breaks are", 
            "specified can produce unpredictable results.", "Please consider using only one or the other.")
    # Rowv is the row dendrogram.
    if (is.null(Rowv) || is.na(Rowv)) 
        Rowv = FALSE
    # Colv is the column dendrogram.
    if (is.null(Colv) || is.na(Colv)) 
        Colv = FALSE
    else if (Colv == "Rowv" && !isTRUE(Rowv)) 
        Colv = FALSE
    # x is the matrix of values to be plotted on the heatmap itself.
    di = dim(x)
    if (length(di) != 2 || !is.numeric(x)) 
        stop("'x' must be a numeric matrix")

    # Number rows and columns in the heatmap matrix.
    nr = di[1]
    nc = di[2]
    if (nr <= 1 || nc <= 1) 
        stop("'x' must have at least 2 rows and 2 columns")

    # margins gives bottom/left/top/right OUTER margins.
    if (!is.numeric(margins) || any(margins < 0))
        stop("'margins' must be a numeric vector with elements >= 0")
    if (length(margins) == 1) 
        margins = c(margins, margins)
    if (length(margins) == 2) 
        margins = c(margins, margins)
    if (length(margins) != 4 || any(is.na(margins)))
        stop("'margins' must be of length 1, 2, or 4")
    mgnNames = c("bottom", "left", "top", "right")
    if (!is.null(names(margins)) && all(names(margins) %in% mgnNames))
        margins = margins[mgnNames]
    names(margins) = mgnNames
    margins = as.list(margins) # So we can index using $

    # heatmapAreaSize gives width and height of heatmap area.
    if (!is.numeric(heatmapAreaSize) || any(heatmapAreaSize < 1))
        stop("'heatmapAreaSize' must be a numeric vector with elements >= 1%")
    if (length(heatmapAreaSize) == 1) 
        heatmapAreaSize = c(heatmapAreaSize, heatmapAreaSize)
    if (length(heatmapAreaSize) != 2 || any(is.na(heatmapAreaSize)))
        stop("'heatmapAreaSize' must be of length 1 or 2")
    # Region 4 row/column sizes are determined by remaining space.
    rgn4Size = c(0,0)
    rgn4Size[1] = 100-(margins$left+heatmapAreaSize[1]+margins$right)
    rgn4Size[2] = 100-(margins$top+heatmapAreaSize[2]+margins$bottom)
    if (any(rgn4Size < 0))
        stop("margins+heatmapAreaSize are > 100%")
    if ((rgn4Size[1] < 1) && (dendrogram %in% c("both", "row")))
        stop("margins+heatmapAreaSize width must be <= 99% when dendro used")
    if ((rgn4Size[2] < 1) && (dendrogram %in% c("both", "column")))
        stop("margins+heatmapAreaSize width must be <= 99% when dendro used")

    # Compute the heatmap color image area boundaries, in percent of heatmap plot area.
    hBnds = list(
        bottom=sum(btmLabSize),
        left=sum(rowSideBarSize) + sum(lftLabSize),
        top=sum(colSideBarSize) + sum(topLabSize),
        right=sum(rgtLabSize))
    if (showDebugInfo)
        cat("hBnds:\n", getPrint(hBnds), "\n")

    # Make sure heatmap component sizes don't exceed 100%.  Add filler at bottom and right
    # to make them equal 100%.
    heatmapFiller = c(0,0)
    heatmapFiller[1] = 100-(hBnds$left+heatmapSize[1]+hBnds$right)
    heatmapFiller[2] = 100-(hBnds$top+heatmapSize[2]+hBnds$bottom)
    if (any(heatmapFiller < 0))
        stop("LabSizes+SideBarSizes+heatmapSize are > 100%")
    hBnds$right = hBnds$right + heatmapFiller[1]
    hBnds$bottom = hBnds$bottom + heatmapFiller[2]
    retval$hBnds = hBnds

    # Compute the dendrogram area margin sizes.  The axis labels are plotted in the margins.
    if (dendrogram %in% c("both", "row"))
        dRowMgns = list(bottom=hBnds$bottom, left=100-sum(rowDendroSize), top=hBnds$top, right=rowDendroSize[2])
    else
        dRowMgns = list(bottom=hBnds$bottom, left=95, top=hBnds$top, right=0)
    if (any(dRowMgns < 0))
        stop("rowDendroSizes are > 100%")
    if (dendrogram %in% c("both", "column"))
        dColMgns = list(bottom=colDendroSize[2], left=hBnds$left, top=100-sum(colDendroSize), right=hBnds$right)
    else
        dColMgns = list(bottom=0, left=hBnds$left, top=95, right=hBnds$right)
    if (any(dColMgns < 0))
        stop("colDendroSizes are > 100%")

    # Compute the key area margin sizes.
    kMgns = list(bottom=100-keySpace[2]-keySize[2], left=keySpace[1], top=keySpace[2], right=100-keySpace[1]-keySize[1])
    if (any(dColMgns < 0))
        stop("keySpace+keySize is > 100%")

   # cellnote is an optional matrix of strings to go in each cell.
    if (missing(cellnote)) 
        cellnote = matrix("", ncol=ncol(x), nrow=nrow(x))
    # Set dendrogram if necessary.
    if (!inherits(Rowv, "dendrogram"))
        {
        if (((!isTRUE(Rowv)) || (is.null(Rowv))) && (dendrogram %in% c("both", "row")))
            {
            if (is.logical(Colv) && (Colv)) 
                dendrogram = "column"
            else dendrogram = "none"
            warning("Discrepancy: Rowv is FALSE, while dendrogram is '", 
                dendrogram, "'. Omitting row dendogram.")
            }
        }
    if (!inherits(Colv, "dendrogram"))
        {
        if (((!isTRUE(Colv)) || (is.null(Colv))) && (dendrogram %in% c("both", "column")))
            {
            if (is.logical(Rowv) && (Rowv)) 
                dendrogram = "row"
            else dendrogram = "none"
            warning("Discrepancy: Colv is FALSE, while dendrogram is '", 
                dendrogram, "'. Omitting column dendogram.")
            }
        }
    # ddr is dendrogram-class object for row dendrogram.  rowInd is leaf labels.
    if (inherits(Rowv, "dendrogram"))
        {
        ddr = Rowv
        rowInd = order.dendrogram(ddr)
        }
    else if (is.integer(Rowv))
        {
        hcr = hclustfun(distfun(x))
        ddr = as.dendrogram(hcr)
        ddr = reorder(ddr, Rowv)
        rowInd = order.dendrogram(ddr)
        if (nr != length(rowInd)) 
            stop("row dendrogram ordering gave index of wrong length")
        }
    else if (isTRUE(Rowv))
        {
        Rowv = rowMeans(x, na.rm=na.rm)
        hcr = hclustfun(distfun(x))
        ddr = as.dendrogram(hcr)
        ddr = reorder(ddr, Rowv)
        rowInd = order.dendrogram(ddr)
        if (nr != length(rowInd)) 
            stop("row dendrogram ordering gave index of wrong length")
        }
    else
        {
        rowInd = nr:1
        }
    # ddc is dendrogram-class object for column dendrogram.  colInd is leaf labels.
    if (inherits(Colv, "dendrogram"))
        {
        ddc = Colv
        colInd = order.dendrogram(ddc)
        }
    else if (identical(Colv, "Rowv"))
        {
        if (nr != nc) 
            stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
        if (exists("ddr"))
            {
            ddc = ddr
            colInd = order.dendrogram(ddc)
            }
        else colInd = rowInd
        }
    else if (is.integer(Colv))
        {
        hcc = hclustfun(distfun(if (symm) x else t(x)))
        ddc = as.dendrogram(hcc)
        ddc = reorder(ddc, Colv)
        colInd = order.dendrogram(ddc)
        if (nc != length(colInd)) 
            stop("column dendrogram ordering gave index of wrong length")
        }
    else if (isTRUE(Colv))
        {
        Colv = colMeans(x, na.rm=na.rm)
        hcc = hclustfun(distfun(if (symm) x else t(x)))
        ddc = as.dendrogram(hcc)
        ddc = reorder(ddc, Colv)
        colInd = order.dendrogram(ddc)
        if (nc != length(colInd)) 
            stop("column dendrogram ordering gave index of wrong length")
        }
    else
        {
        colInd = 1:nc
        }
    retval$rowInd = rowInd
    retval$colInd = colInd
    retval$call = match.call()
    # Subset the plotted data to only that subset that is part of the dendrograms.
    x = x[rowInd, colInd]
    x.unscaled = x
    cellnote = cellnote[rowInd, colInd]
    # labRow is row names of dendrogram/heatmap, and colLabRow is colors for them.
    if (is.null(labRow)) 
        labRow = if (is.null(rownames(x))) (1:nr)[rowInd] else rownames(x)
    else labRow = labRow[rowInd]
    if (length(colLabRow) < length(rowInd))
        colLabRow = c(colLabRow, rep("black", length(rowInd)-length(colLabRow)))
    colLabRow = colLabRow[rowInd]
    # labCol is column names of dendrogram/heatmap, and colLabCol is colors for them.
    if (is.null(labCol))
        labCol = if (is.null(colnames(x))) (1:nc)[colInd] else colnames(x)
    else labCol = labCol[colInd]
    if (length(colLabCol) < length(colInd))
        colLabCol = c(colLabCol, rep("black", length(colInd)-length(colLabCol)))
    colLabCol = colLabCol[colInd]
    # lftLabs, rghtLabs, btmLabs, topLabs are the labels for the different sides
    # of the dendrograms/heatmap.
    if (is.null(lftLabs))
        lftLabs = labRow
    else lftLabs = lftLabs[rowInd]
    if (is.null(rgtLabs))
        rgtLabs = labRow
    else rgtLabs = rgtLabs[rowInd]
    if (is.null(btmLabs))
        btmLabs = labCol
    else btmLabs = btmLabs[colInd]
    if (is.null(topLabs))
        topLabs = labCol
    else topLabs = topLabs[colInd]
    # Scale the heatmap values if so requested.  Unscaled values are in x.unscaled.
    if (scale == "row")
        {
        retval$rowMeans = rm = rowMeans(x, na.rm=na.rm)
        x = sweep(x, 1, rm)
        retval$rowSDs = sx = apply(x, 1, sd, na.rm=na.rm)
        x = sweep(x, 1, sx, "/")
        }
    else if (scale == "column")
        {
        retval$colMeans = rm = colMeans(x, na.rm=na.rm)
        x = sweep(x, 2, rm)
        retval$colSDs = sx = apply(x, 2, sd, na.rm=na.rm)
        x = sweep(x, 2, sx, "/")
        }
    # Get decent color bins if there are none.
    if (missing(breaks) || is.null(breaks) || length(breaks) < 1)
        {
        if (missing(col) || is.function(col)) 
            breaks = 16
        else breaks = length(col) + 1
        }
    if (length(breaks) == 1)
        {
        if (!symbreaks) 
            breaks = seq(min(x, na.rm=na.rm), max(x, na.rm=na.rm), length=breaks)
        else
            {
            extreme = max(abs(x), na.rm=TRUE)
            breaks = seq(-extreme, extreme, length=breaks)
            }
        }
    nbr = length(breaks)
    # ncol is number of colors.  Get colors if a color function was provided.
    ncol = length(breaks) - 1
    if (class(col) == "function") 
        col = col(ncol)
    min.breaks = min(breaks)
    max.breaks = max(breaks)
    # Force heatmap values to limits if they lie outside the color bin limits.
    x[x < min.breaks] = min.breaks
    x[x > max.breaks] = max.breaks

    # Layout stuff.  The layout:
    #  Cell 1: Heatmap:
    #                        ColLabels
    #                        ColSideBar
    #   RowLabels RowSideBar Heatmap
    #  Cell 2: y-axis label and Row dendrogram to its right
    #  Cell 3: x-axis label and Column dendrogram below it
    #  Cell 4: Key:
    #                KeyTitle
    # ylabKey y-axis KeyColorImage
    #                   x-axis
    #                   xlabKey
    # If lmat wasn't specified, create it and lhei and lwid.  If it was specified, don't
    # touch it, assume user has set lmat/lhei/lwid properly.  lmat is a matrix indicating
    # which plot element appears where on the page.  If the lmat entry is 1, for example,
    # this means that the heatmap will show up in that matrix panel.
    if (missing(lmat) || is.null(lmat))
        {
        lmat = matrix(c(4,3,2,1), nrow=2, byrow=TRUE)
        # lwid is the width of the layout columns.
        lwid = c(rgn4Size[1], heatmapAreaSize[1])
        # lhei is the height of the layout rows.  Units are arbitrary (relative to each
        # other), but here we make them be percent of display area width/height.
        lhei = c(rgn4Size[2], heatmapAreaSize[2])
        # Don't allow any row or column sizes to be < 0.1.
        lhei[lhei < 0.1] = 0.1
        lwid[lwid < 0.1] = 0.1
        }
    # Check sizes of lhei/lwid.
    if (length(lhei) != nrow(lmat)) 
        stop("lhei must have length = nrow(lmat) = ", nrow(lmat))
    if (length(lwid) != ncol(lmat)) 
        stop("lwid must have length = ncol(lmat) =", ncol(lmat))
    # Show lmat/lhei/lwid and margins if debug.
    if (showDebugInfo)
        {
        cat("lmat:\n", getPrint(lmat), "\n")
        cat("lhei: ", getPrint(lhei), "\n")
        cat("lwid: ", getPrint(lwid), "\n")
        cat("Margins: ", getPrint(margins), "\n")
        }

    # Get settable par parameters and save them for restoration on exit.
    op = par(no.readonly=TRUE)
    on.exit(par(op))
    
    # Set outer margins.  Set cex before setting omi, so that oma will reflect that cex.
    # Use the larger of cex.main and cex.sub for this.
    par(cex=max(cex.main, cex.sub))
    par(omi=c(pctToInch(h=margins$bottom), pctToInch(w=margins$left),
        pctToInch(h=margins$top), pctToInch(w=margins$right)))
    
    # Process the layout matrix and initiate a new plot.  This actually controls the
    # plotting device.  We move from one panel to another with plotting commands that
    # start a new plot.  So here, we start with panel 1 = heatmap (if present).
    layout(lmat, widths=lwid, heights=lhei, respect=FALSE)

    # Plot cell #1: heatmap with optional color side bars and labels all around.

    # First set par(mai) to 0.  We use 0 margins for the heatmap, because image() has
    # trouble plotting within the margins, even when margin clipping is turned off.
    # So, we'll compute the proper plot position for each item manually without relying
    # on margins.
    par(xaxs='i', yaxs='i', mai=c(0,0,0,0))

    # Plot the heatmap itself.
    if (showDebugInfo)
        cat("Plotting heatmap\n")
    x = t(x)
    cellnote = t(cellnote)
    if (revC) # Reverse column order?
        {
        iy = nr:1
        if (exists("ddr")) 
            ddr = rev(ddr)
        x = x[, iy]
        cellnote = cellnote[, iy]
        }
    else iy = 1:nr
    # We want the heatmap itself to be located at (0,0) through (nc,nr).  To accomplish
    # this, we need to calculate what the xlim and ylim limits would be, when margins are
    # 0, to place the heatmap at the position given by hBnds.
    sf = c(nc,nr)/heatmapSize
    xlim = c(-hBnds$left*sf[1], nc+hBnds$right*sf[1])
    ylim = c(-hBnds$bottom*sf[2], nr+hBnds$top*sf[2])
    plot(NA, type="n", xlim=xlim, ylim=ylim, ann=FALSE, axes=FALSE, frame.plot=showDebugInfo)
    image(x=0:nc, y=0:nr, z=x, add=TRUE, axes=FALSE, xlab="", ylab="", col=col, breaks=breaks, cex=1, fill)
    retval$carpet = x
    if (exists("ddr")) 
        retval$rowDendrogram = ddr
    if (exists("ddc")) 
        retval$colDendrogram = ddc
    retval$breaks = breaks
    retval$col = col
    # na.color is the color to use for NA entries.
    # if (!invalid(na.color) & any(is.na(x))) # What is invalid() ???
    if (any(is.na(x)))
        {
        mmat = ifelse(is.na(x), 1, NA)
        image(x=1:nc, y=1:nr, z=mmat, axes=FALSE, xlab="", ylab="", col=na.color, add=TRUE)
        }
    # Column separator boxes.
    if (!missing(colsep)) 
        for (csep in colsep)
            rect(xleft=csep-1-sepwidth[1]/2, ybottom=0, xright=csep-1+sepwidth[1]/2, ytop=nr,
                col=sepcolor, border=NA)

    # Row separator boxes.
    if (!missing(rowsep)) 
        for (rsep in rowsep)
            rect(xleft=0, ybottom=rsep-1-sepwidth[2]/2, xright=nc, ytop=rsep-1+sepwidth[2],
                col=sepcolor, border=NA)

    # Make a "trace" line in the heatmap if requested.
    min.scale = min(breaks)
    max.scale = max(breaks)
    x.scaled = scale01(t(x), min.scale, max.scale)

    # Column trace.
    if (trace %in% c("both", "column"))
        {
        retval$vline = vline
        vline.vals = scale01(vline, min.scale, max.scale)
        for (i in colInd)
            {
            if (!is.null(vline))
                lines(x=rep(i-1+vline.vals,2), y=c(0,nr), col=linecol, lwd=1, lty=2)
            xv = rep(i, nrow(x.scaled)) + x.scaled[,i] - 1
            xv = c(xv[1], xv)
            yv = 1:length(xv)-1
            lines(x=xv, y=yv, lwd=1, col=tracecol, type="s")
            }
        }

    # Row trace.
    if (trace %in% c("both", "row"))
        {
        retval$hline = hline
        hline.vals = scale01(hline, min.scale, max.scale)
        for (i in rowInd)
            {
            if (!is.null(hline))
                lines(x=c(0,nc), y=rep(i-1+hline.vals,2), col=linecol, lwd=1, lty=2)
            yv = rep(i, ncol(x.scaled)) + x.scaled[i, ] - 1
            yv = rev(c(yv[1], yv))
            xv = length(yv):1-1
            lines(x=xv, y=yv, lwd=1, col=tracecol, type="s")
            }
        }

    # Text within each heatmap cell.
    if (!missing(cellnote)) 
        text(x=c(row(cellnote))-0.5, y=c(col(cellnote))-0.5, labels=c(cellnote), 
            col=notecol, cex=notecex)

    # Frame around the heatmap.
    if (frame)
        rect(xleft=0, ybottom=0, xright=nc, ytop=nr, col=NA, border=col.frame, lwd=lwd.frame)

    # Plot row side bar in left margin.  Save user X-position of left side of side bar,
    # or of left side of heatmap if no sidebar, for use below in plotting row labels.
    xLeft = 0 # x-position of left edge of heatmap in user coords.
    if (!is.null(RowSideColors))
        {
        # x-position of right side of row sidebar is just left of the heatmap position,
        # so it is par("usr")[1] - rowSideBarSize[2], except that the first of those is
        # user coordinates and the second is pct of heatmap area.  Calculate it in user
        # coords.  The left side of the row sidebar follows immediately and simply.
        x2 = xLeft - pctToUser(w=rowSideBarSize[2])
        x1 = x2 - pctToUser(w=rowSideBarSize[1])
        image(x=c(x1, x2), y=0:nr, z=rbind(1:nr), add=TRUE, col=RowSideColors[rowInd], axes=FALSE)
        xLeft = x1
        }

    # Likewise, plot column side bar in top margin.
    yTop = nr # y-position of top edge of heatmap in user coords.
    if (!is.null(ColSideColors))
        {
        y1 = yTop + pctToUser(h=colSideBarSize[2])
        y2 = y1 + pctToUser(h=colSideBarSize[1])
        image(x=0:nc, y=c(y1, y2), z=cbind(1:nc), add=TRUE, col=rep(ColSideColors[colInd]), axes=FALSE)
        yTop = y2
        }

    # Plot row labels left of xLeft.
    if (is.null(cexRow))
        cexRow=1/log10(nr)
    if (lftLabRow)
        {
        # Similar to row side bar above.
        x2 = xLeft - pctToUser(w=lftLabSize[2])
        x1 = x2 - pctToUser(w=lftLabSize[1])
        text(x2, iy-0.5, lftLabs, pos=2, cex=cexRow, col=colLabRow, srt=srtLabRow)
        }

    # Plot row labels right of heat map.
    if (is.null(cexCol))
        cexCol=1/log10(nc)
    if (rgtLabRow)
        {
        # Similar to column side bar above.
        xRight = nc # x-position of right edge of heatmap in user coords.
        x1 = xRight + pctToUser(w=rgtLabSize[2])
        text(x1, iy-0.5, rgtLabs, pos=4, cex=cexRow, col=colLabRow, srt=srtLabRow)
        }

    # Plot column labels above yTop.
    if (topLabCol)
        {
        # Similar to row labels right above.
        y1 = yTop + pctToUser(h=topLabSize[2])
        # This is not right, but it seems to do a pretty good job.
        ang = srtLabCol*pi/180
        sinang = sin(ang)
        cosang = cos(ang)
        sinpluscosang = sinang+cosang
        adj = c((1-sinang*sinpluscosang)/2, (1-cosang*sinpluscosang)/2)
        text((1:nc)-0.5, y1, topLabs, adj=adj, cex=cexCol, col=colLabCol, srt=srtLabCol)
        }

    # Plot column labels below heat map.
    if (btmLabCol)
        {
        # Similar to row labels right above.
        yBelow = 0 # y-position of bottom edge of heatmap in user coords.
        y2 = yBelow - pctToUser(h=btmLabSize[2])
        # This is not right, but it seems to do a pretty good job.
        ang = srtLabCol*pi/180
        sinang = sin(ang)
        cosang = cos(ang)
        sinpluscosang = sinang+cosang
        adj = c((1+sinang*sinpluscosang)/2, (1+cosang*sinpluscosang)/2)
        text((1:nc)-0.5, y2, btmLabs, adj=adj, cex=cexCol, col=colLabCol, srt=srtLabCol)
        }

    # Call user plot function if there is one.
    if (!is.null(plot.Heatmap))
        plot.Heatmap(xlim, ylim, nr, nc)

    # Before plotting in plot cells #2, 3, and 4, define a function to plot the key, which
    # may be in any one of those plot cells depending on the value of "key".
    plotKey = function(keyRgnSize)
        {
        if (showDebugInfo)
            cat("Plotting key, plt=", par("plt"), "kMgns:\n", getPrint(kMgns), "\n")

        # First set par(mai) appropriately from kMgns.
        par(xaxs='r', yaxs='r', cex=cex.key)
        par(mai=c(pctToInch(h=kMgns$bottom, pct2=keyRgnSize[2]),
                  pctToInch(w=kMgns$left, pct2=keyRgnSize[1]),
                  pctToInch(h=kMgns$top, pct2=keyRgnSize[2]),
                  pctToInch(w=kMgns$right, pct2=keyRgnSize[1])))
        plot.window(xlim=c(0,1), ylim=c(0,1))
        par(cex=cex.key)
        sv = par(xpd=TRUE)
        clip(0,1,0,1)
        if (showDebugInfo)
            cat(" now plt=", par("plt"), "\n")

        # Compute key's bins.  This has been problematic.  The plot span is x=0,1.  We
        # want the key image in that area.  That means the center points of the leftmost
        # and rightmost bins are not at 0 and 1, but inside of that by half of 1/N.  The
        # breakpoints are the edges of the bins, so these DO include values located at
        # 0 and 1.  keybreaks has one more element than number of colors, and gives the
        # points at which color begins (and ends).
        keybreaks = breaks
        if (symkey)
            {
            max.raw = max(abs(c(x, breaks)), na.rm=TRUE)
            min.raw = -max.raw
            keybreaks[1] = min.raw
            keybreaks[length(keybreaks)] = max.raw
            }
        else
            {
            min.raw = min(c(x, breaks), na.rm=TRUE)
            max.raw = max(c(x, breaks), na.rm=TRUE)
            }

        # Key x-axis.
        if (is.null(lvKey))
            lvKey = pretty(breaks)
        while ((lvKey[1] < min.raw) && length(lvKey) > 2)
            lvKey = lvKey[-1]
        while ((lvKey[length(lvKey)] > max.raw) && length(lvKey) > 2)
            lvKey = lvKey[-length(lvKey)]
        # lvKey values are values at centers of bins, so first and last are not at x=0,1 but
        # inside of that by 1/2n, i.e. min.raw is not at x=0 but at x=1/2*length(lvKey).  We
        # must compute actual values at x=0,1 and pass those to scale01.
        dx = (max.raw-min.raw)/(2*(length(lvKey)-1))
        xv = scale01(as.numeric(lvKey), min.raw-dx, max.raw+dx)
        if (is.null(xvalKey))
            xvalKey = lvKey
        axis(1, at=xv, labels=xvalKey, cex.axis=cex.axisKey, lwd=0, lwd.tick=1)

        # Draw the key color map.  It needs to go from x=0 to x=1.  Default x,y args for
        # image() are supposed to use equally-spaced values in range (0,1) but this
        # appears not to be working correctly.  The problem was that when x arg to image()
        # is not specified, default x has length = length(col), and therefore it specifies
        # midpoints of cells, not boundaries of cells like you want.
        xz = seq(0, 1, length=length(col)+1)
        z = seq(min.raw, max.raw, length=length(col))
        image(x=xz, z=matrix(z, ncol=1), col=col, add=TRUE, xaxt='n', yaxt='n', breaks=keybreaks)

        # Frame around key image.
        if (frame)
            rect(xleft=0, xright=1, ybottom=0, ytop=1, col=NA, border=col.frame, lwd=lwd.frame-1)
        # Key x-axis label.
        if (is.null(xlabKey))
            {
            if (scale == "row") 
                xlabKey = "Row Z-Score"
            else if (scale == "column") 
                xlabKey = "Column Z-Score"
            else
                xlabKey = "Value"
            }
        mtext(side=1, xlabKey, line=line.xlabKey, cex=cex.xlabKey)

        # Density-type key.
        if (density.info == "density")
            {
            dens = density(x, adjust=densadj, na.rm=TRUE)
            omit = dens$x < min(breaks) | dens$x > max(breaks)
            dens$x = dens$x[-omit]
            dens$y = dens$y[-omit]
            dens$x = scale01(dens$x, min.raw, max.raw)
            # Draw density curve.
            lines(dens$x, dens$y/max(dens$y) * 0.95, col=denscol, lwd=1)
            # Key y-axis.
            axis(2, at=pretty(dens$y)/max(dens$y) * 0.95, pretty(dens$y), cex.axis=cex.axisKey)
            # Key y-axis label.
            if (is.null(ylabKey))
                ylabKey = "Density"
            # Key title.
            if (is.null(mainKey))
                mainKey = "Color Key and Density Plot"
            }
        # Histogram-type key.
        else if (density.info == "histogram")
            {
            # Draw histogram.
            h = hist(x, plot=FALSE, breaks=breaks)
            # Draw curve.
            hx = scale01(breaks, min.raw, max.raw)
            hy = c(h$counts, h$counts[length(h$counts)])
            lines(hx, hy/max(hy) * 0.95, lwd=1, type="s", col=denscol)
            # Key y-axis.
            axis(2, at=pretty(hy)/max(hy) * 0.95, pretty(hy), cex.axis=cex.axisKey)
            # Key y-axis label.
            if (is.null(ylabKey))
                ylabKey = "Count"
            # Key title.
            if (is.null(mainKey))
                mainKey = "Color Key and Histogram"
            }
        else
            {
            if (is.null(mainKey))
                mainKey = "Color Key"
            if (is.null(ylabKey))
                ylabKey = ""
            }
        title(mainKey, line=0.5, cex.main=cex.mainKey)
        mtext(side=2, ylabKey, line=line.ylabKey, cex=cex.ylabKey)
        par(xpd=sv)
        }

    # Plot title and subtitle.  This can't be done before plotting plot cell #1 because it
    # requires plot.new(), which messes things up with layout().
    if (!is.null(main))
        mtext(main, side=3, line=line.main, outer=TRUE, cex=cex.main, col=col.main, font=2)
    if (!is.null(sub)) 
        mtext(sub, side=3, line=line.sub, outer=TRUE, cex=cex.sub, col=col.sub, font=2)

    # Plot cell #2: Row dendrogram and y-axis label.

    # First set par(mai) appropriately from dRowMgns.
    par(xaxs='i', yaxs='i')
    par(mai=c(pctToInch(h=dRowMgns$bottom, pct2=heatmapAreaSize[2]),
              pctToInch(w=dRowMgns$left, pct2=rgn4Size[1]),
              pctToInch(h=dRowMgns$top, pct2=heatmapAreaSize[2]),
              pctToInch(w=dRowMgns$right, pct2=rgn4Size[1])))

    # Plot the row dendrogram if it was requested.
     if (showDebugInfo)
        cat("Plotting row dendrogram\n")
    if (dendrogram %in% c("both", "row"))
        {
        sv = par(xpd=NA)
        plot(ddr, horiz=TRUE, axes=FALSE, leaflab="none", edge.root=TRUE)
        par(sv)
        }
    else
        plot(NA, type="n", xlim=c(0,100), ylim=c(0,nc), ann=FALSE, axes=FALSE, frame.plot=showDebugInfo)

    # Plot the y-axis label if it was requested.
    if (!is.null(ylab)) 
        mtext(ylab, side=2, line=line.xlab, cex=cex.ylab, col=col.ylab)

    # Plot the key if it was requested.
    if (key == 2)
        plotKey(c(rgn4Size[1], heatmapAreaSize[2]))

    # Plot cell #3: Column dendrogram and x-axis label.

    # First set par(mai) appropriately from dColMgns.
    par(xaxs='i', yaxs='i')
    par(mai=c(pctToInch(h=dColMgns$bottom, pct2=rgn4Size[2]),
              pctToInch(w=dColMgns$left, pct2=heatmapAreaSize[1]),
              pctToInch(h=dColMgns$top, pct2=rgn4Size[2]),
              pctToInch(w=dColMgns$right, pct2=heatmapAreaSize[1])))

    # Plot the column dendrogram if it was requested.
     if (showDebugInfo)
        cat("Plotting column dendrogram\n")
    if (dendrogram %in% c("both", "column"))
        {
        sv = par(xpd=NA)
        plot(ddc, horiz=FALSE, axes=FALSE, leaflab="none", edge.root=TRUE)
        par(sv)
        }
    else
        plot(NA, type="n", xlim=c(0,nr), ylim=c(0,100), ann=FALSE, axes=FALSE, frame.plot=showDebugInfo)

    # Plot the x-axis label if it was requested.
    if (!is.null(xlab)) 
        mtext(xlab, side=3, line=line.ylab, cex=cex.xlab, col=col.xlab)

    # Plot the key if it was requested.
    if (key == 3)
        plotKey(c(heatmapAreaSize[1], rgn4Size[2]))

    # Plot cell #4: Key if key == TRUE == 1 or key == 4.

    # Set margins to nothing so plot() will succeed.
    par(mai=c(0,0,0,0))
    plot(NA, type="n", xlim=c(0,1), ylim=c(0,1), ann=FALSE, axes=FALSE, frame.plot=showDebugInfo)

    # Plot the key if it was requested.
    if (key == 1 || key == 4)
        plotKey(rgn4Size)

    # Return a bunch of stuff.
    retval$colorTable = data.frame(low=retval$breaks[-length(retval$breaks)], 
        high=retval$breaks[-1], color=retval$col, stringsAsFactors=FALSE)

    # Restore options.
    par(op)
    return(invisible(retval))
    }

#######################################################################################
# Make dendrograms and/or a heatmap of the data in data frame df.  The data frame must
# have row and column names, which are the labels of the plots.  Dendrograms may be
# provided in rowDendro and/or colDendro, or if these are NA the dendrograms are
# computed by making a distance matrix from dataframe df, but if they are NULL, the
# row and/or column dendrogram is not computed and shown.  If dendrogram is not NA, it
# specifies which (if any) dendrograms are to be actually drawn, but if NA, it is set
# to draw those dendrograms that are computed or supplied.  The distance matrix is
# computed using either Euclidean distance (default), 1 - abs(Pearson correlation),
# or sign(Pearson correlation) * [1 - abs(Pearson correlation)].
# Plots are made if a title is specified, and are skipped when the title is NA.
# Colors may be specified in the heatmapColors argument (blues/yellows for default).
# If valueForNA is not NA, all NAs in df are replaced by valueForNA when making the
# heatmap (but not when making the dendrograms; there, they are ignored).
#
# Returns: list (invisibly) with these elements:
#   row: row dendrogram
#   col: column dendrogram
#   others: see return values for tedsHeatmap().
#######################################################################################
makeHeatmap = function(df, titleRowDendrogram=NA, titleColumnDendrogram=NA,
    titleHeatmap=NA, subtitleHeatmap="", showDebugInfo=FALSE,
    valueForNA=NA, rowDendro=NULL, colDendro=NULL, dendrogram=NA,
    cexRow=0.5, cexCol=0.6, cexLabKey=0.6, cex.main=0.8, cex.sub=0.7,
    distance=c("Euclidean", "absPearson", "signedPearson"), # First one is the default
    heatmapColors=makeHeatmapColors("black", "yellow", 127, "blue", 127),
    ...)
    {
    # Make distance matrices for hierarchical clustering below.  agnes()
    # doesn't accept NA values, so change these to very large distances.
    if (distance[1] == "Euclidean")
        {
        dfRow = dist(df)
        dfCol = dist(t(df))
        }
    else if (distance[1] == "absPearson")
        {
        dfRow = 1 - abs(cor(t(df), use="complete.obs"))
        dfCol = 1 - abs(cor(df, use="complete.obs"))
        }
    else if (distance[1] == "signPearson")
        {
        dfRow = cor(t(df), use="complete.obs")
        dfRow = sign(dfRow) * (1 - abs(dfRow))
        dfCol = cor(df, use="complete.obs")
        dfCol = sign(dfCol) * (1 - abs(dfCol))
        }
    else stop("Invalid distance value:", distance)
    dfRow[is.na(dfRow)] = 1e99
    dfCol[is.na(dfCol)] = 1e99

    # Do hierarchical clustering of distance matrices.
    if (!is.null(rowDendro))
        {
        if (all(is.na(rowDendro)))
            {
            rowDendro = agnes(dfRow, diss=TRUE)
            if (!is.na(titleRowDendrogram))
                plot(rowDendro, which.plots=2, main=titleRowDendrogram)
            }
        rowDendro = as.dendrogram(as.hclust(rowDendro))
        }
    if (!is.null(colDendro))
        {
        if (all(is.na(colDendro)))
            {
            colDendro = agnes(dfCol, diss=TRUE)
            if (!is.na(titleColumnDendrogram))
                plot(colDendro, which.plots=2, cex=0.5, main=titleColumnDendrogram)
            }
        colDendro = as.dendrogram(as.hclust(colDendro))
        }
    if (is.na(dendrogram))
        {
        dendrogramChoices = c("none", "row", "column", "both")
        choiceValue = 1+(!is.null(rowDendro))+(2*!is.null(colDendro))
        dendrogram = dendrogramChoices[choiceValue]
        }

    # Replace NA values if requested.
    if (!is.na(valueForNA))
        df = apply(df, 2, function(r) { r[is.na(r)] = valueForNA; return(r) })

    # Now plot the heat map if requested.
    L = list()
    if (!is.na(titleHeatmap))
        L = tedsHeatmap(as.matrix(df), Rowv=rowDendro, Colv=colDendro, trace="none",
            dendrogram=dendrogram, cexRow=cexRow, cexCol=cexCol,
            cex.xlabKey=cexLabKey, cex.ylabKey=cexLabKey, col=heatmapColors,
            cex.main=cex.main, main=titleHeatmap, cex.sub=cex.sub, sub=subtitleHeatmap,
            showDebugInfo=showDebugInfo, ...)
    L$row=rowDendro
    L$col=colDendro
    return(invisible(L))
    }

#######################################################################################
# Test tedsHeatmap().
#######################################################################################
test.tedsHeatmap = function(useDefaults=FALSE, useMakeHeatmap=FALSE)
    {
    requirePackage("gplots")
    data(mtcars)
    x = as.matrix(mtcars)
    nr = nrow(x)
    nc = ncol(x)
    rc = rainbow(nr, start=0, end=.3)
    cc = rainbow(nc, start=0, end=.3)
    cellnote = matrix(1:(nr*nc), nrow=nr)
    if (useDefaults)
        {
        tedsHeatmap(x, dendrogram="both", col=heat.colors(256), scale="column", trace="none", #"both",
            xlab="specification variables", ylab="Car Models",
            cellnote=cellnote, main="heatmap(<Mtcars data>)", sub="Subheading",
            colsep=2:nc, rowsep=2:nr, sepcolor="black",
            RowSideColors=rc, ColSideColors=cc,
            lftLabRow=TRUE, rgtLabRow=TRUE, btmLabCol=TRUE, topLabCol=TRUE)
        }
    else if (useMakeHeatmap)
        {
        makeHeatmap(x, titleHeatmap="heatmap(<Mtcars data>)")
        }
    else
        {
        tedsHeatmap(x, dendrogram="both", margins=c(5,10,10,10), heatmapAreaSize=c(40,70),
            heatmapSize=c(40,80), col=heat.colors(256), scale="column", trace="none", #"both",
            xlab="specification variables", ylab="Car Models", cexRow=0.5, cexCol=0.6,
            cellnote=cellnote, notecex=0.3, notecol="black",
            rowDendroSize=c(90,0), colDendroSize=c(80,0),
            main="heatmap(<Mtcars data>)", sub="Subheading", cex.sub=0.9,
            colsep=2:nc, rowsep=2:nr, sepcolor="black",
            RowSideColors=rc, ColSideColors=cc, rowSideBarSize=c(5,1), colSideBarSize=c(5,1),
            lftLabSize=c(25,1), rgtLabSize=c(25,1), btmLabSize=c(5,1), topLabSize=c(5,1),
            lftLabRow=TRUE, rgtLabRow=TRUE, btmLabCol=TRUE, topLabCol=TRUE)
        }
    }

#######################################################################################
# End of file.
#######################################################################################
Sourced.Include_Heatmaps = TRUE
cat("  Include_Heatmaps.R included\n")
